"use strict";
var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Endpoint_instances, _Endpoint_setupCorsHeaders, _Endpoint_parseOutput, _Endpoint_runMiddlewares, _Endpoint_parseAndRunHandler, _Endpoint_handleResult;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Endpoint = exports.AbstractEndpoint = void 0;
const zod_1 = require("zod");
const helpers_1 = require("./helpers");
class AbstractEndpoint {
    getDescription() {
        return this.description;
    }
}
exports.AbstractEndpoint = AbstractEndpoint;
/** mIN, OPT - from Middlewares */
class Endpoint extends AbstractEndpoint {
    constructor({ middlewares, inputSchema, outputSchema, handler, resultHandler, description, ...rest }) {
        super();
        _Endpoint_instances.add(this);
        this.methods = [];
        this.middlewares = [];
        this.middlewares = middlewares;
        this.inputSchema = helpers_1.combineEndpointAndMiddlewareInputSchemas(inputSchema, middlewares);
        this.outputSchema = outputSchema;
        this.handler = handler;
        this.resultHandler = resultHandler;
        this.description = description;
        if ('methods' in rest) {
            this.methods = rest.methods;
        }
        else {
            this.methods = [rest.method];
        }
    }
    getMethods() {
        return this.methods;
    }
    getInputSchema() {
        return this.inputSchema;
    }
    getOutputSchema() {
        return this.outputSchema;
    }
    getPositiveResponseSchema() {
        return this.resultHandler.getPositiveResponse(this.outputSchema).schema;
    }
    getNegativeResponseSchema() {
        return this.resultHandler.getNegativeResponse().schema;
    }
    getPositiveMimeTypes() {
        return this.resultHandler.getPositiveResponse(this.outputSchema).mimeTypes;
    }
    getNegativeMimeTypes() {
        return this.resultHandler.getNegativeResponse().mimeTypes;
    }
    async execute({ request, response, logger, config }) {
        let output;
        let error = null;
        if (config.cors) {
            __classPrivateFieldGet(this, _Endpoint_instances, "m", _Endpoint_setupCorsHeaders).call(this, response);
        }
        if (request.method === 'OPTIONS') {
            return response.end();
        }
        const initialInput = helpers_1.getInitialInput(request);
        try {
            const { input, options, isStreamClosed } = await __classPrivateFieldGet(this, _Endpoint_instances, "m", _Endpoint_runMiddlewares).call(this, {
                input: { ...initialInput },
                request, response, logger
            });
            if (isStreamClosed) {
                return;
            }
            output = __classPrivateFieldGet(this, _Endpoint_instances, "m", _Endpoint_parseOutput).call(this, await __classPrivateFieldGet(this, _Endpoint_instances, "m", _Endpoint_parseAndRunHandler).call(this, { input, options, logger }));
        }
        catch (e) {
            error = e;
        }
        await __classPrivateFieldGet(this, _Endpoint_instances, "m", _Endpoint_handleResult).call(this, {
            initialInput, output, request,
            response, error, logger
        });
    }
}
exports.Endpoint = Endpoint;
_Endpoint_instances = new WeakSet(), _Endpoint_setupCorsHeaders = function _Endpoint_setupCorsHeaders(response) {
    const accessMethods = this.methods.map((method) => method.toUpperCase()).concat('OPTIONS').join(', ');
    response.set('Access-Control-Allow-Origin', '*');
    response.set('Access-Control-Allow-Methods', accessMethods);
    response.set('Access-Control-Allow-Headers', 'content-type');
}, _Endpoint_parseOutput = function _Endpoint_parseOutput(output) {
    try {
        return this.outputSchema.parse(output);
    }
    catch (e) {
        if (e instanceof zod_1.z.ZodError) {
            throw new zod_1.z.ZodError([
                {
                    message: 'Invalid format',
                    code: 'custom',
                    path: ['output'],
                },
                ...e.issues.map((issue) => ({
                    ...issue,
                    path: issue.path.length === 0 ? ['output'] : issue.path
                }))
            ]);
        }
        throw e;
    }
}, _Endpoint_runMiddlewares = async function _Endpoint_runMiddlewares({ input, request, response, logger }) {
    const options = {};
    for (const def of this.middlewares) {
        input = { ...input, ...def.input.parse(input) }; // middleware can transform the input types
        Object.assign(options, await def.middleware({
            input, options, request,
            response, logger
        }));
        if (response.writableEnded) {
            break;
        }
    }
    return { input, options, isStreamClosed: response.writableEnded };
}, _Endpoint_parseAndRunHandler = async function _Endpoint_parseAndRunHandler({ input, options, logger }) {
    return await this.handler({
        input: this.inputSchema.parse(input),
        options, logger
    });
}, _Endpoint_handleResult = async function _Endpoint_handleResult({ error, request, response, logger, initialInput, output }) {
    try {
        await this.resultHandler.handler({
            error, output, request, response, logger,
            input: initialInput
        });
    }
    catch (e) {
        logger.error(`Result handler failure: ${e.message}.`);
        // throw e;
    }
};
//# sourceMappingURL=endpoint.js.map