import { z } from 'zod';
import { Endpoint, Handler } from './endpoint';
import { ApiResponse, FlatObject, IOSchema, Merge } from './helpers';
import { Method, MethodsDefinition } from './method';
import { MiddlewareDefinition } from './middleware';
import { ResultHandlerDefinition } from './result-handler';
declare type BuildProps<IN extends IOSchema, OUT extends IOSchema, mIN, mOUT, M extends Method> = {
    input: IN;
    output: OUT;
    handler: Handler<z.output<Merge<IN, mIN>>, z.input<OUT>, mOUT>;
    description?: string;
} & MethodsDefinition<M>;
/** mIN, mOUT - accumulated from all middlewares */
export declare class EndpointsFactory<mIN, mOUT, POS extends ApiResponse, NEG extends ApiResponse> {
    #private;
    protected resultHandler: ResultHandlerDefinition<POS, NEG>;
    protected middlewares: MiddlewareDefinition<any, any, any>[];
    constructor(resultHandler: ResultHandlerDefinition<POS, NEG>);
    addMiddleware<IN extends IOSchema, OUT extends FlatObject>(definition: MiddlewareDefinition<IN, mOUT, OUT>): EndpointsFactory<Merge<IN, mIN>, mOUT & OUT, POS, NEG>;
    build<IN extends IOSchema, OUT extends IOSchema, M extends Method>({ input, output, handler, description, ...rest }: BuildProps<IN, OUT, mIN, mOUT, M>): Endpoint<IN, OUT, mIN, mOUT, M, POS, NEG>;
}
export declare const defaultEndpointsFactory: EndpointsFactory<unknown, unknown, ApiResponse<z.ZodObject<{
    status: z.ZodLiteral<"success">;
    data: import("./helpers").OutputMarker;
}, "strip", z.ZodTypeAny, {
    data: {
        [x: string]: any;
        [x: number]: any;
    };
    status: "success";
}, {
    data: {
        [x: string]: any;
        [x: number]: any;
    };
    status: "success";
}>>, ApiResponse<z.ZodObject<{
    status: z.ZodLiteral<"error">;
    error: z.ZodObject<{
        message: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        message: string;
    }, {
        message: string;
    }>;
}, "strip", z.ZodTypeAny, {
    status: "error";
    error: {
        message: string;
    };
}, {
    status: "error";
    error: {
        message: string;
    };
}>>>;
export {};
