"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ZodFile = void 0;
const zod_1 = require("zod");
const zodFileKind = 'ZodFile';
// the copy of the private Zod errorUtil.errToObj
const errToObj = (message) => typeof message === 'string' ? { message } : message || {};
const base64Regex = /^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=)?$/;
class ZodFile extends zod_1.ZodType {
    constructor() {
        super(...arguments);
        this.binary = (message) => new ZodFile({
            ...this._def,
            checks: [
                ...this._def.checks,
                { kind: 'binary', ...errToObj(message) },
            ],
        });
        this.base64 = (message) => new ZodFile({
            ...this._def,
            checks: [
                ...this._def.checks,
                { kind: 'base64', ...errToObj(message) },
            ],
        });
    }
    _parse(ctx, data, parsedType) {
        if (parsedType !== zod_1.ZodParsedType.string) {
            ctx.addIssue(data, {
                code: zod_1.ZodIssueCode.invalid_type,
                expected: zod_1.ZodParsedType.string,
                received: parsedType,
            });
            return zod_1.INVALID;
        }
        let invalid = false;
        for (const check of this._def.checks) {
            if (check.kind === 'base64') {
                if (!base64Regex.test(data)) {
                    invalid = true;
                    ctx.addIssue(data, {
                        code: zod_1.ZodIssueCode.custom,
                        message: check.message,
                    });
                }
            }
        }
        return invalid ? zod_1.INVALID : zod_1.OK(data);
    }
    get isBinary() {
        return !!this._def.checks.find((check) => check.kind === 'binary');
    }
    get isBase64() {
        return !!this._def.checks.find((check) => check.kind === 'base64');
    }
}
exports.ZodFile = ZodFile;
ZodFile.create = () => new ZodFile({
    checks: [],
    typeName: zodFileKind
});
//# sourceMappingURL=file-schema.js.map