import { Request } from 'express';
import { z } from 'zod';
import { LoggerConfig } from './config-type';
import { MiddlewareDefinition } from './middleware';
export declare type FlatObject = Record<string, any>;
declare type ObjectSchema = z.AnyZodObject;
declare type Extractable = 'shape' | '_unknownKeys' | '_catchall' | '_output' | '_input';
declare type UnionSchema = z.ZodUnion<[ObjectSchema, ...ObjectSchema[]]>;
declare type IntersectionSchema = z.ZodIntersection<ObjectSchema, ObjectSchema>;
export declare type IOSchema = ObjectSchema | UnionSchema | IntersectionSchema;
export declare type ArrayElement<T extends readonly unknown[]> = T extends readonly (infer K)[] ? K : never;
declare type UnionResult<T extends ObjectSchema[], F extends Extractable> = ArrayElement<T>[F];
declare type IntersectionResult<T extends IntersectionSchema, F extends Extractable> = T['_def']['left'][F] & T['_def']['right'][F];
declare type IOExtract<T extends IOSchema | any, F extends Extractable> = T extends ObjectSchema ? T[F] : T extends UnionSchema ? UnionResult<T['options'], F> : T extends IntersectionSchema ? IntersectionResult<T, F> : unknown;
export declare type Merge<A extends IOSchema, B extends IOSchema | any> = z.ZodObject<IOExtract<A, 'shape'> & IOExtract<B, 'shape'>, IOExtract<A, '_unknownKeys'>, IOExtract<A, '_catchall'>, IOExtract<A, '_output'> & IOExtract<B, '_output'>, IOExtract<A, '_input'> & IOExtract<B, '_input'>>;
export declare type OutputMarker = IOSchema & {
    _typeGuard: 'OutputMarker';
};
export declare const markOutput: (output: IOSchema) => OutputMarker;
export declare type ReplaceMarkerInShape<S extends z.ZodRawShape, OUT extends IOSchema> = {
    [K in keyof S]: S[K] extends OutputMarker ? OUT : S[K];
};
export declare function extractObjectSchema(subject: IOSchema): ObjectSchema;
export declare function combineEndpointAndMiddlewareInputSchemas<IN extends IOSchema, mIN>(input: IN, middlewares: MiddlewareDefinition<IOSchema, any, any>[]): Merge<IN, mIN>;
export declare function getInitialInput(request: Request): any;
export declare function isLoggerConfig(logger: any): logger is LoggerConfig;
export declare function getMessageFromError(error: Error): string;
export declare function getStatusCodeFromError(error: Error): number;
export declare type ApiResponse<A = z.ZodTypeAny> = {
    schema: A;
    mimeTypes: string[];
};
export declare const createApiResponse: <S extends z.ZodTypeAny>(schema: S, mimeTypes?: string | string[]) => ApiResponse<S>;
export {};
