"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createApiResponse = exports.getStatusCodeFromError = exports.getMessageFromError = exports.isLoggerConfig = exports.getInitialInput = exports.combineEndpointAndMiddlewareInputSchemas = exports.extractObjectSchema = exports.markOutput = void 0;
const http_errors_1 = require("http-errors");
const mime_1 = require("mime");
const zod_1 = require("zod");
const config_type_1 = require("./config-type");
const markOutput = (output) => output;
exports.markOutput = markOutput;
function extractObjectSchema(subject) {
    if (subject instanceof zod_1.z.ZodUnion) {
        return subject.options.reduce((acc, option) => acc.partial().merge(option.partial()));
    }
    if (subject instanceof zod_1.z.ZodIntersection) {
        return subject._def.left.merge(subject._def.right);
    }
    return subject;
}
exports.extractObjectSchema = extractObjectSchema;
function combineEndpointAndMiddlewareInputSchemas(input, middlewares) {
    if (middlewares.length === 0) {
        return extractObjectSchema(input);
    }
    const mSchema = middlewares
        .map((middleware) => middleware.input)
        .reduce((carry, schema) => extractObjectSchema(carry).merge(extractObjectSchema(schema)));
    return extractObjectSchema(mSchema).merge(extractObjectSchema(input));
}
exports.combineEndpointAndMiddlewareInputSchemas = combineEndpointAndMiddlewareInputSchemas;
function getInitialInput(request) {
    switch (request.method) {
        case 'POST':
        case 'PUT':
        case 'PATCH':
            return request.body;
        case 'GET':
            return request.query;
        case 'DELETE': // _may_ have body
        default:
            return { ...request.query, ...request.body };
    }
}
exports.getInitialInput = getInitialInput;
function isLoggerConfig(logger) {
    return typeof logger === 'object' &&
        'level' in logger && 'color' in logger &&
        Object.keys(config_type_1.loggerLevels).includes(logger.level) &&
        typeof logger.color === 'boolean';
}
exports.isLoggerConfig = isLoggerConfig;
function getMessageFromError(error) {
    return error instanceof zod_1.z.ZodError
        ? error.issues.map(({ path, message }) => `${path.join('/')}: ${message}`).join('; ')
        : error.message;
}
exports.getMessageFromError = getMessageFromError;
function getStatusCodeFromError(error) {
    if (error instanceof http_errors_1.HttpError) {
        return error.statusCode;
    }
    if (error instanceof zod_1.z.ZodError) {
        return 400;
    }
    return 500;
}
exports.getStatusCodeFromError = getStatusCodeFromError;
const createApiResponse = (schema, mimeTypes = mime_1.getType('json')) => {
    return {
        schema,
        mimeTypes: typeof mimeTypes === 'string' ? [mimeTypes] : mimeTypes,
    };
};
exports.createApiResponse = createApiResponse;
//# sourceMappingURL=helpers.js.map