import { Request, Response } from 'express';
import { Logger } from 'winston';
import { z } from 'zod';
import { ApiResponse, IOSchema } from './helpers';
interface ResultHandlerParams<RES> {
    error: Error | null;
    input: any;
    output: any;
    request: Request;
    response: Response<RES>;
    logger: Logger;
}
declare type ResultHandler<RES> = (params: ResultHandlerParams<RES>) => void | Promise<void>;
export interface ResultHandlerDefinition<POS extends ApiResponse, NEG extends ApiResponse> {
    getPositiveResponse: <OUT extends IOSchema>(output: OUT) => POS;
    getNegativeResponse: () => NEG;
    handler: ResultHandler<z.output<POS['schema']> | z.output<NEG['schema']>>;
}
export declare const createResultHandler: <POS extends ApiResponse<z.ZodTypeAny>, NEG extends ApiResponse<z.ZodTypeAny>>(definition: ResultHandlerDefinition<POS, NEG>) => ResultHandlerDefinition<POS, NEG>;
export declare const defaultResultHandler: ResultHandlerDefinition<ApiResponse<z.ZodObject<{
    status: z.ZodLiteral<"success">;
    data: import("./helpers").OutputMarker;
}, "strip", z.ZodTypeAny, {
    data: {
        [x: string]: any;
        [x: number]: any;
    };
    status: "success";
}, {
    data: {
        [x: string]: any;
        [x: number]: any;
    };
    status: "success";
}>>, ApiResponse<z.ZodObject<{
    status: z.ZodLiteral<"error">;
    error: z.ZodObject<{
        message: z.ZodString;
    }, "strip", z.ZodTypeAny, {
        message: string;
    }, {
        message: string;
    }>;
}, "strip", z.ZodTypeAny, {
    status: "error";
    error: {
        message: string;
    };
}, {
    status: "error";
    error: {
        message: string;
    };
}>>>;
export {};
