"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultResultHandler = exports.createResultHandler = void 0;
const zod_1 = require("zod");
const helpers_1 = require("./helpers");
const createResultHandler = (definition) => definition;
exports.createResultHandler = createResultHandler;
exports.defaultResultHandler = exports.createResultHandler({
    getPositiveResponse: (output) => helpers_1.createApiResponse(zod_1.z.object({
        status: zod_1.z.literal('success'),
        data: helpers_1.markOutput(output)
    })),
    getNegativeResponse: () => helpers_1.createApiResponse(zod_1.z.object({
        status: zod_1.z.literal('error'),
        error: zod_1.z.object({
            message: zod_1.z.string()
        })
    })),
    handler: ({ error, input, output, request, response, logger }) => {
        if (!error) {
            response.status(200).json({
                status: 'success',
                data: output
            });
            return;
        }
        const statusCode = helpers_1.getStatusCodeFromError(error);
        if (statusCode === 500) {
            logger.error(`Internal server error\n${error.stack}\n`, {
                url: request.url,
                payload: input
            });
        }
        response.status(statusCode).json({
            status: 'error',
            error: { message: helpers_1.getMessageFromError(error) }
        });
    }
});
//# sourceMappingURL=result-handler.js.map