"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createServer = exports.attachRouting = void 0;
const express_1 = __importDefault(require("express"));
const helpers_1 = require("./helpers");
const logger_1 = require("./logger");
const result_handler_1 = require("./result-handler");
const routing_1 = require("./routing");
const http_errors_1 = __importDefault(require("http-errors"));
function attachRouting(config, routing) {
    const logger = helpers_1.isLoggerConfig(config.logger) ? logger_1.createLogger(config.logger) : config.logger;
    return routing_1.initRouting({ app: config.app, routing, logger, config });
}
exports.attachRouting = attachRouting;
function createServer(config, routing) {
    const logger = helpers_1.isLoggerConfig(config.logger) ? logger_1.createLogger(config.logger) : config.logger;
    const app = express_1.default();
    const errorHandler = config.errorHandler || result_handler_1.defaultResultHandler;
    const jsonParser = config.server.jsonParser || express_1.default.json();
    const jsonFailureHandler = (error, request, response, next) => {
        if (!error) {
            return next();
        }
        errorHandler.handler({
            error, request, response, logger,
            input: request.body,
            output: null
        });
    };
    const lastResortHandler = (request, response) => {
        errorHandler.handler({
            request, response, logger,
            error: http_errors_1.default(404, `Can not ${request.method} ${request.path}`),
            input: null,
            output: null
        });
    };
    app.use([jsonParser, jsonFailureHandler]);
    routing_1.initRouting({ app, routing, logger, config });
    app.use(lastResortHandler);
    return app.listen(config.server.listen, () => {
        logger.info(`Listening ${config.server.listen}`);
    });
}
exports.createServer = createServer;
//# sourceMappingURL=server.js.map